'use strict';

/**
 * Checks if value is array.
 * @param v value
 */
function is(v) {
    return Array.isArray(v);
}

/**
 * Gets zero-based index.
 * @param x an array
 * @param i index (-ve: from right) (0)
 */
function index(x, i = 0) {
    return i < 0 ? Math.max(x.length + i, 0) : Math.min(i, x.length);
}

/**
 * Sets value at index.
 * @param x an array (updated)
 * @param i index
 * @param v value
 * @returns x
 */
function set$(x, i, v) {
    x[index(x, i)] = v;
    return x;
}

/**
 * Gets last value.
 * @param x an array
 * @param vd default value
 */
function last(x, vd) {
    return x.length > 0 ? x[x.length - 1] : vd;
}

/**
 * Gets value at path in a nested array.
 * @param x a nested array
 * @param p path
 */
function getPath(x, p) {
    for (var i of p)
        x = is(x) ? x[i] : undefined;
    return x;
}

/**
 * Sets value at path in a nested array.
 * @param x a nested array (updated)
 * @param p path
 * @param v value
 * @returns x
 */
function setPath$(x, p, v) {
    var y = getPath(x, p.slice(0, -1));
    if (is(y))
        set$(y, last(p), v);
    return x;
}

module.exports = setPath$;
